﻿using AsReaderP252BiOSDemo.Platforms.iOS;
using AsReaderP252BSDK;

namespace AsReaderP252BiOSDemo;

public partial class MainPage : ContentPage
{
    private AsReaderP252BMAUIManager asReaderP252BMAUIManager;
    private AsReaderP252BSDelegate asReaderP252BDelegate;
    private List<AsReaderP252B> arrDeviceList;

    public MainPage()
    {
        InitializeComponent();
        BluetoothSearchButton.Text = "Search";
        arrDeviceList = new List<AsReaderP252B>();
        asReaderP252BMAUIManager = AsReaderP252BMAUIManager.sharedInstance();
        asReaderP252BDelegate = AsReaderP252BSDelegate.sharedInstance();
    }
    protected override void OnAppearing()
    {
        base.OnAppearing();
        if (asReaderP252BMAUIManager.isConnect == true)
        {
            reloadUI(true);
        }
        else
        {
            reloadUI(false);
        }
        asReaderP252BDelegate.WhenAsReaderP252BIsConnected = (bool isConnected) =>
        {
            WhenAsReaderP252BIsConnected(isConnected);
        };
        asReaderP252BDelegate.OnAsReaderP252BTriggerKeyEvent = (bool status) =>
        {
            return false;
        };
        asReaderP252BDelegate.DidDiscoverAsReaderP252BDeviceList = (List<AsReaderP252B> list) =>
        {
            DidDiscoverAsReaderP252BDeviceList(list);
        };
    }
    private void WhenAsReaderP252BIsConnected(bool isConnected)
    {
        if (isConnected)
        {
            asReaderP252BMAUIManager.StopDiscovery();
            MainThread.BeginInvokeOnMainThread(() =>
            {
                StartActivityIndicator("");
                reloadUI(true);
            });
        }
        else
        {
            reloadUI(false);
        }
    }
    private void DidDiscoverAsReaderP252BDeviceList(List<AsReaderP252B> list)
    {
        arrDeviceList = list;
    }
    private void reloadUI(bool status)
    {
        MainThread.BeginInvokeOnMainThread(() =>
        {
            if (status == true)
            {
                APPVersionLabel.Text = asReaderP252BMAUIManager.GetAPPVersion();
                DLLVersionLabel.Text = asReaderP252BMAUIManager.GetAsReaderP25BSDKDLL();
                SDKVersionLabel.Text = asReaderP252BMAUIManager.GetSDKVersion();
                FirmwareVersionLabel.Text = asReaderP252BMAUIManager.GetFirmwareVersion();
                UHFLabel.Text = asReaderP252BMAUIManager.GetRFModuleVersion();
                SNLabel.Text = asReaderP252BMAUIManager.GetSerialNumber();
                RegionLabel.Text = asReaderP252BMAUIManager.GetRegion();
                CheckBattery(asReaderP252BMAUIManager.GetBatteryStatus());
                BarocdeScanLabel.TextColor = Colors.Black;
                RFIDInventoryLabel.TextColor = Colors.Black;
                LogoImage.Source = "asreader_log_on.png";
                StopActivityIndicator();
            }
            else
            {
                APPVersionLabel.Text = "";
                DLLVersionLabel.Text = "";
                SDKVersionLabel.Text = "";
                FirmwareVersionLabel.Text = "";
                UHFLabel.Text = "";
                SNLabel.Text = "";
                RegionLabel.Text = "";
                BatteryImage.Source = "battery0.png";
                BarocdeScanLabel.TextColor = Colors.LightGray;
                RFIDInventoryLabel.TextColor = Colors.LightGray;
                LogoImage.Source = "asreader_log_off.png";
            }
        });
    }
    private void CheckBattery(int battery)
    {
        switch (battery)
        {
            case 0:
                BatteryImage.Source = "battery0.png";
                break;
            case 1:
                BatteryImage.Source = "battery1.png";
                break;
            case 2:
                BatteryImage.Source = "battery2.png";
                break;
            case 3:
                BatteryImage.Source = "battery3.png";
                break;
            case 4:
                BatteryImage.Source = "battery4.png";
                break;
            default:
                return;
        }
    }
    [Obsolete]
    public void BluetoothSearchButton_Clicked(object sender, EventArgs e)
    {
        Device.StartTimer(TimeSpan.FromSeconds(10.0), showSegment);
        StartActivityIndicator("");
        arrDeviceList.Clear();
        asReaderP252BMAUIManager.StartDiscovery();
    }
    private bool showSegment()
    {
        MainThread.BeginInvokeOnMainThread(() =>
        {
            if (arrDeviceList.Count > 0)
            {
                showAlert(title: "Search for bluetooth", message: "", bleDevices: arrDeviceList);
            }
            else
            {
                DisplayAlert("No reader found.", "Check the power of the reader.", "OK");
            }
            StopActivityIndicator();
        });
        return false;
    }
    async void BarocdeScanClicked(object sender, EventArgs e)
    {
        if (asReaderP252BMAUIManager.isConnect)
        {
            await Navigation.PushAsync(new BarcodeScanPage());
        }
    }
    async void RFIDInventoryClicked(object sender, EventArgs e)
    {
        if (asReaderP252BMAUIManager.isConnect)
        {
            await Navigation.PushAsync(new RFIDInventoryPage());
        }
    }
    private async void showAlert(string title, string message, List<AsReaderP252B> bleDevices)
    {
        List<string> list = new List<string>();
        Dictionary<string, AsReaderP252B> dictionary = new Dictionary<string, AsReaderP252B>();
        for (int i = 0; i < (int)bleDevices.Count; i++)
        {
            AsReaderP252B device = bleDevices[i];
            list.Add(device.DeviceName);
            dictionary.Add(device.DeviceName, device);
        }
        string[] buttons = list.ToArray();
        var name = await DisplayActionSheet(title, "Cancel", null, buttons);
        if (dictionary.ContainsKey(name))
        {
            AsReaderP252B asReaderP252B = dictionary[name];
            asReaderP252BMAUIManager.ConnectDevice(asReaderP252B);
        }
    }
    private void StartActivityIndicator(string message)
    {
        asReaderP252BMAUIManager.ShowGlobalProgressHUDWithTitle(message);
        BluetoothSearchButton.IsEnabled = false;
    }
    private void StopActivityIndicator()
    {
        asReaderP252BMAUIManager.DismissGlobalHUD();
        BluetoothSearchButton.IsEnabled = true;
    }
}


