﻿using System;
using System.Collections;
using System.Collections.ObjectModel;
using AsReaderP252BiOSDemo.Platforms.iOS;
using AsReaderP252BiOSSDK;
using Microsoft.Maui;

namespace AsReaderP252BiOSDemo;

public partial class RFIDInventoryPage : ContentPage
{
    private AsReaderP252BMAUIManager asReaderP252BMAUIManager;
    private AsReaderP252BSDelegate asReaderP252BDelegate;
    ArrayList powerGainArray;
    int count;
    int powerGain;
    private AsReaderP252BCommandType actionState;
    private ObservableCollection<RFIDData> RFIDItems = new ObservableCollection<RFIDData>();
    public ObservableCollection<RFIDData> ItemData { get { return RFIDItems; } }
    private Dictionary<string, RFIDData> dataDic = new Dictionary<string, RFIDData>();

    public RFIDInventoryPage()
	{
		InitializeComponent();
        count = 0;
        powerGain = 5;
        actionState = AsReaderP252BCommandType.Stop;
        asReaderP252BMAUIManager = AsReaderP252BMAUIManager.sharedInstance();
        asReaderP252BDelegate = AsReaderP252BSDelegate.sharedInstance();
    }

    protected override void OnAppearing()
    {
        base.OnAppearing();
        StartActivityIndicator();
        RFIDListView.ItemsSource = RFIDItems;
        asReaderP252BMAUIManager.SetScanMode(AsReaderP252BScanMode.RFIDScanMode);
        asReaderP252BDelegate.WhenAsReaderP252BIsConnected = (bool isConnected) =>
        {
            WhenAsReaderP252BIsConnected(isConnected);
        };
        asReaderP252BDelegate.OnAsReaderP252BTriggerKeyEvent = (bool status) =>
        {
            bool sta = OnAsReaderP252BTriggerKeyEvent(status);
            return sta;
        };
        asReaderP252BDelegate.ReadAsReaderP252BTag = (string tag, float rssi, float phase, float frequency) =>
        {
            ReadAsReaderP252BTag(tag, rssi, phase, frequency);
        };
        asReaderP252BDelegate.ChangedAsReaderP252BActionState = (AsReaderP252BCommandType action, nint resultCode) =>
        {
            ChangedAsReaderP252BActionState(action, resultCode);
        };
      
        Task.Run(() =>
        {
            try
            {
                powerGainArray = asReaderP252BMAUIManager.GetPowerGainRange();
                Thread.Sleep(500);
                powerGain = asReaderP252BMAUIManager.PowerGain;
            }
            catch (Exception)
            {
                powerGain = asReaderP252BMAUIManager.GetPowerRangeMax();
            }
            MainThread.BeginInvokeOnMainThread(() =>
            {
                RescanTimePicker.Title = (powerGain / 10).ToString() + ".0dB";
                if (powerGainArray != null)
                {
                    foreach (string power in powerGainArray)
                    {
                        RescanTimePicker.Items.Add(power);
                    }
                }
                if (RescanTimePicker.Title == "")
                {
                    RescanTimePicker.SelectedIndex = 0;
                }
                else
                {
                    RescanTimePicker.SelectedIndex = powerGainArray.IndexOf(RescanTimePicker.Title);
                }
                StopActivityIndicator();
            });
        });
    }
    protected override void OnDisappearing()
    {
        base.OnDisappearing();
        asReaderP252BMAUIManager.Stop();
    }
    void StartScanButton_Clicked(object sender, EventArgs e)
    {
        if (actionState == AsReaderP252BCommandType.Inventory)
        {
            asReaderP252BMAUIManager.Stop();
        }
        else if (actionState == AsReaderP252BCommandType.Stop)
        {
            asReaderP252BMAUIManager.Inventory();
        }
    }
    void ClearScanButton_Clicked(object sender, EventArgs e)
    {
        count = 0;
        CountLabel.Text = count.ToString();
        dataDic.Clear();
        RFIDItems.Clear();
    }
    void OnPickerSelectedIndexChanged(object sender, EventArgs e)
    {
        var picker = (Picker)sender;
        int selectedIndex = picker.SelectedIndex;
        if (selectedIndex != -1)
        {
            string powerGain = picker.Items[selectedIndex];
            RescanTimePicker.Title = powerGain;
            selectedIndex = selectedIndex + asReaderP252BMAUIManager.GetPowerRangeMin() / 10;
            asReaderP252BMAUIManager.PowerGain = selectedIndex * 10;
        }
    }
    private void WhenAsReaderP252BIsConnected(bool isConnected)
    {
        MainThread.BeginInvokeOnMainThread(() =>
        {
            if (!isConnected)
            {
                Navigation.PopAsync();
            }
        });
    }
    private void ReadAsReaderP252BTag(string tag, float rssi, float phase, float frequency)
    {
        RFIDData InventoryDataChanged;
        if (dataDic.ContainsKey(tag))
        {
            InventoryDataChanged = dataDic[tag];
            InventoryDataChanged.Count++;
            dataDic[tag] = InventoryDataChanged;
        }
        else
        {
            int nowCount = dataDic.Count;
            InventoryDataChanged = new RFIDData(tag, 1);
            dataDic.Add(tag, InventoryDataChanged);
            RFIDItems.Add(InventoryDataChanged);
        }
        count++;
        MainThread.BeginInvokeOnMainThread(() =>
        {
            RFIDListView.ItemsSource = null;
            RFIDListView.ItemsSource = RFIDItems;
            CountLabel.Text = count.ToString();
        });
    }
    private bool OnAsReaderP252BTriggerKeyEvent(bool status)
    {
        MainThread.BeginInvokeOnMainThread(() =>
        {
            if (status)
            {
                asReaderP252BMAUIManager.Inventory();
            }
            else
            {
                asReaderP252BMAUIManager.Stop();
            }
        });
        return false;
    }
    private void ChangedAsReaderP252BActionState(AsReaderP252BCommandType action, nint resultCode)
    {
        MainThread.BeginInvokeOnMainThread(() =>
        {
            actionState = action;
            if (action == AsReaderP252BCommandType.Inventory)
            {
                StartScanButton.Text = "Stop Scan";
            }
            else if (action == AsReaderP252BCommandType.Stop)
            {
                StartScanButton.Text = "Inventory";
            }
        });
    }
    public void StartActivityIndicator()
    {
        asReaderP252BMAUIManager.ShowGlobalProgressHUDWithTitle("");
        RFIDListView.IsEnabled = false;
        StartScanButton.IsEnabled = false;
        ClearScanButton.IsEnabled = false;
        RescanTimePicker.IsEnabled = false;
    }
    public void StopActivityIndicator()
    {
        asReaderP252BMAUIManager.DismissGlobalHUD();
        RFIDListView.IsEnabled = true;
        StartScanButton.IsEnabled = true;
        ClearScanButton.IsEnabled = true;
        RescanTimePicker.IsEnabled = true;
    }
}
